@@ -32,17 +32,17 @@ IP:10.64.105.64
 host:osc-mon-d116.mr.jkcont
 """
 
-import vxi11Device
-import time,types,struct,exceptions
+import cVXI11
+import time,types,struct,sys
 import array,struct,numpy
 
-class WP700Z(vxi11Device.Vxi11Device):
-    def __init__(self, device, timeout=5):
+class WP700Z(cVXI11.IEEE488_2Device):
+    def __init__(self, device="inst0", timeout=5):
         #visa.Instrument.__init__(self, device, timeout=timeout)
-        vxi11Device.Vxi11Device.__init__(self, host=device, device="inst0")
+        cVXI11.Vxi11Device.__init__(self, host=device, device=device)
         self.device=device
         self.timeout=timeout
-        self.write("CHDR OFF") # always header off
+        self.write(b"CHDR OFF") # always header off
         self.IDN_Str=self.IDN()
         ID=self.IDN_Str[:-1].split(",")
         self.Make=ID[0]
@@ -50,7 +50,7 @@ class WP700Z(vxi11Device.Vxi11Device):
         self.Option=ID[3]
 
     def IDN(self):
-        return self.ask("*IDN?;")
+        return self.ask(b"*IDN?;")
 
     def respToDict(self, lables,resp):
         v=resp[:-1].split(";")
@@ -59,38 +59,44 @@ class WP700Z(vxi11Device.Vxi11Device):
         return d
 
     def qALST(self):
-        return self.ask("ALST?")
+        return self.ask(b"ALST?")
 
-    def ATTN(self,ch="C1",att=1):
+    def ATTN(self,ch=b"C1",att=1):
         """
         <channel> : = {C1, C2, C3, C4, EX, EX10}
         <attenuation> : = {1, 2, 5, 10, 20, 25, 50, 100, 200,500, 1000, 10000}
         """
-        return self.write("%s:ATTN %d;"%(ch,att))
+        _channels= ("C1", "C2", "C3", "C4", "EX", "EX10")
+        _attenuations=(1, 2, 5, 10, 20, 25, 50, 100, 200,500, 1000, 10000)
+        if ch not in _channels:
+            raise ValueError("Channel name shoudl be one of {chs}".format(chs=_channels))
+        if att not in _attenuations:
+            raise ValueError("Attenuation value shoudl be one of {atts}".format(atts=_attenuations))
+        return self.write(b"%s:ATTN %d;"%(ch,att))
 
-    def qATTN(self,ch="C1"):
-        return self.ask("%s:ATTN?;"%ch)
+    def qATTN(self,ch=b"C1"):
+        return self.ask(b"%s:ATTN?;"%ch)
 
-    def ACAL(self, state="ON"):
+    def ACAL(self, state=b"ON"):
         """<state> : = {ON, OFF}"""
-        return self.write("ACAL %s;"%state)
+        return self.write(b"ACAL %s;"%state)
 
     def qACAL(self):
-        return self.ask("ACAL?;")
+        return self.ask(b"ACAL?;")
 
-    def ASET(self, ch="C1",FIND=False):
+    def ASET(self, ch=b"C1",FIND=False):
         """
         <channel> : = {C1, C2, C3, C4}
         """
         if FIND:
-            self.write("%s:ASET FIND"%ch)
+            self.write(b"%s:ASET FIND"%ch)
         else:
-            self.write("%s:ASET"%ch)
+            self.write(b"%s:ASET"%ch)
             
     def qBWL(self):
-        return self.ask("BWL?")
+        return self.ask(b"BWL?")
     
-    def BWL(self,mode="OFF"):
+    def BWL(self,mode=b"OFF"):
         """
         BandWidth_Limit <mode>
         BandWidth_Limit <channel>,<mode>[,<channel>,<mode>[,<channel>,<mode>[,<channel>,<mode>]]]
@@ -98,22 +104,22 @@ class WP700Z(vxi11Device.Vxi11Device):
         Note: OFF = Full, ON = 20MHz
         <channel> : = {C1, C2, C3, C4}
         """
-        return self.write("BLW %s"%mode)
+        return self.write(b"BLW %s"%mode)
     
     def ARM(self):
-        self.ask("ARM;")
+        self.ask(b"ARM;")
         
-    def BUZZ(slef,state="BEEP"):
+    def BUZZ(slef,state=b"BEEP"):
         """
         <state>:= {BEEP, ON, OFF}
         """
-        return self.write("BUZZ %s;"%state)
+        return self.write(b"BUZZ %s;"%state)
 
 
     def qCAL(self):
-        return self.ask("*CAL?")
+        return self.ask(b"*CAL?")
 
-    def COUT(self,mode,level="",rate="",width=""):
+    def COUT(self,mode,level=b"",rate=b"",width=b""):
         """
         Cal_OUTput <mode>[,<level>[,<rate>]]
         Cal_OUTput PULSE[,<width>]
@@ -121,31 +127,31 @@ class WP700Z(vxi11Device.Vxi11Device):
         <level> : = 5 mV to 1.00 V into 1 MOhm
         <rate> : = 5 Hz to 5 MHz.
         """
-        if mode == "PULSE":
+        if mode == b"PULSE":
             if width:
-                return self.write("COUT PULSE,%s"%width)
+                return self.write(b"COUT PULSE,%s"%width)
             else:
-                return self.write("COUT PULSE;")
+                return self.write(b"COUT PULSE;")
         else:
             if rate:
-                return self.write("COUT %s,%s,%s"%(mode,level,rate))
+                return self.write(b"COUT %s,%s,%s"%(mode,level,rate))
             elif level:
-                return self.write("COUT %s,%s"%(mode,level))
+                return self.write(b"COUT %s,%s"%(mode,level))
             else:
-                return self.write("COUT %s"%mode)
+                return self.write(b"COUT %s"%mode)
             
     def CLM(self,memory):
         """
         CLear_Memory < memory>
         <memory> : = {M1, M2, M3, M4}
         """
-        return self.write("CLM %s"%memory)
+        return self.write(b"CLM %s"%memory)
 
     def CLSW(self):
         """
         CLear SWeeps
         """
-        return self.write("CLSW;")
+        return self.write(b"CLSW;")
     
     def qCMR(self):
         """
@@ -164,15 +170,15 @@ class WP700Z(vxi11Device.Vxi11Device):
         12 EOI detected during definite length data block transfer
         13 Extra bytes detected during definite length data block transfer
         """
-        return self.ask("CMR?;")
+        return self.ask(b"CMR?;")
 
-    def COMB(self, state="AUTO"):
+    def COMB(self, state=b"AUTO"):
         """
         COMBine_channels <state>
         <state> : = {1, 2, AUTO}
         Selecting 1 means no combining of channels will take place; i.e., inthe Timebase (Horizontal) dialog, 4 channels will be shown as theselection.
         """
-        return self.write("COMB %s"%state)
+        return self.write(b"COMB %s"%state)
     
 
     def CFMT(self, block_format, data_type, encoding):
@@ -182,16 +188,16 @@ class WP700Z(vxi11Device.Vxi11Device):
         <data_type> : = {BYTE, WORD}
         <encoding> : = {BIN}
         """
-        return self.write("CFMT %s,%s,%s"%(block_format, data_type, encoding))
+        return self.write(b"CFMT %s,%s,%s"%(block_format, data_type, encoding))
     
     def reconnect(self):
-        vxi11Device.Vxi11Device.__init__(self, host=self.device,
+        cVXI11.Vxi11Device.__init__(self, host=self.device,
                                          device="inst0,0")
         #visa.Instrument.__init__(self, self.device, timeout=self.timeout)
-        self.write("CHDR OFF") # always header off
+        self.write(b"CHDR OFF") # always header off
 
         self.IDN_Str=self.IDN()
-        ID=self.IDN_Str[:-1].split(",")
+        ID=self.IDN_Str[:-1].split(b",")
         self.Make=ID[0]
         self.Model=ID[1]
         self.Option=ID[3]
@@ -199,33 +205,33 @@ class WP700Z(vxi11Device.Vxi11Device):
     def time_div(self, value=None, tonum=False):
         if value==None:
             if tonum==True:
-                return float(self.ask("TDIV?"))
+                return float(self.ask(b"TDIV?"))
             else:
-                return self.ask("TDIV?")
+                return self.ask(b"TDIV?")
         else:
             try:
-                self.write("TDIV %f" % value)
+                self.write(b"TDIV %f" % value)
             except TypeError:
-                self.write("TDIV %s" % value)
+                self.write(b"TDIV %s" % value)
 
     def volt_div(self, ch=1, value=None, tonum=False):
         trace=("C1","C2","C3","C4", "TA", "TB", "TC","TD",
                "M1","M2","M3","M4")
         if (ch in trace):
-            chstr="%s" % ch
+            chstr=b"%s" % (ch.encode()) # note that bytes does not have .format method.
         elif (ch in (1,2,3,4)):
-            chstr="C%d" % ch
+            chstr=b"C%d" % ch
 
         if value==None:
             if tonum==True:
-                return float(self.ask("%s:VDIV?"%chstr))
+                return float(self.ask(b"%s:VDIV?"%chstr))
             else:
-                return self.ask("%s:VDIV?"%chstr)
+                return self.ask(b"%s:VDIV?"%chstr)
         else:
             try:
-                self.write("%s:VDIV %f" % (chstr,value))
+                self.write(b"%s:VDIV %f" % (chstr,value))
             except TypeError:
-                self.write("%s:VDIV %s" % (chstr,value))
+                self.write(b"%s:VDIV %s" % (chstr,value))
 
     def offset(self, ch=1, value=None, tonum=False):
         trace=("C1","C2","C3","C4", "TA", "TB", "TC","TD",
@@ -236,48 +242,48 @@ class WP700Z(vxi11Device.Vxi11Device):
             chstr="C%d" % ch
         if value==None:
             if tonum==True:
-                return float(self.ask("%s:OFST?"%chstr))
+                return float(self.ask(b"%s:OFST?"%chstr))
             else:
-                return self.ask("%s:OFST?"%chstr)
+                return self.ask(b"%s:OFST?"%chstr)
         else:
             try:
-                self.write("%s:OFST %f" % (chstr,value))
+                self.write(b"%s:OFST %f" % (chstr,value))
             except TypeError:
-                self.write("OFST %s" % (chstr,value))
+                self.write(b"OFST %s" % (chstr,value))
 
     def trig_delay(self, value=None, tonum=False):
         if value==None:
             if tonum==True:
-                return float(self.ask("TRDL?"))
+                return float(self.ask(b"TRDL?"))
             else:
-                return self.ask("TRDL?")
+                return self.ask(b"TRDL?")
         else:
             try:
-                self.write("TRDL %f" % value)
+                self.write(b"TRDL %f" % value)
             except TypeError:
-                self.write("TRDL %s" % value)
+                self.write(b"TRDL %s" % value)
 
     def trig_mode(self, mode=None):
         trig=("AUTO","NORM","SINGLE","STOP")
         if mode==None:
-            return self.ask("TRMD?")
+            return self.ask(b"TRMD?")
         else:
             if (mode in trig):
-                self.write("TRMD %s" % mode)
+                self.write(b"TRMD %s" % mode)
             elif isinstance(mode, int):
-                self.write("TRMD %s" % trig[mode])
+                self.write(b"TRMD %s" % trig[mode])
 
     def memory_size(self):
         """mem size set is not yet implemented yet"""
-        return self.ask("MSIZ?")
+        return self.ask(b"MSIZ?")
 
     def comm_order(self):
         """comm order set set is not yet implemented yet"""
-        return self.ask("CORD?")
+        return self.ask(b"CORD?")
 
     def sequence(self):
         # MSIZE 
-        return self.ask("SEQ?")
+        return self.ask(b"SEQ?")
 
     def get_waveform(self, ch, np=0, fp=0, sp=0):
         """
@@ -291,26 +297,26 @@ class WP700Z(vxi11Device.Vxi11Device):
         trace=("C1","C2","C3","C4", "TA", "TB", "TC","TD",
                "M1","M2","M3","M4")
         if (ch in trace):
-            self.write("WFSU NP, %d, FP, %d, SP, %d" % (np,fp,sp))
-            return self.ask("%s:WAVEFORM?" % ch)
+            self.write(b"WFSU NP, %d, FP, %d, SP, %d" % (np,fp,sp))
+            return self.ask(b"%s:WAVEFORM?" % ch)
         elif (ch in (1,2,3,4)):
-            self.write("WFSU NP, %d, FP, %d, SP, %d" % (np,fp,sp))
-            return self.ask("C%d:WAVEFORM?" % ch)
+            self.write(b"WFSU NP, %d, FP, %d, SP, %d" % (np,fp,sp))
+            return self.ask(b"C%d:WAVEFORM?" % ch)
 
     def Run(self):
-        self.write(":ACQ:STATE RUN;")
+        self.write(b":ACQ:STATE RUN;")
 
     def Stop(self):
-        self.write(":ACQ:STATE STOP;")
+        self.write(b":ACQ:STATE STOP;")
 
     def clear(self):
-        self.write("*CLS;")
+        self.write(b"*CLS;")
         
     def wait(self):
-        self.write("*WAI;")
+        self.write(b"*WAI;")
         
     def device_clear(self,flags=0,lock_timeout=0,io_timeout=5):
-        vxi11Device.Vxi11Device.clear(self, flags, lock_timeout, io_timeout)
+        cVXI11.Vxi11Device.clear(self, flags, lock_timeout, io_timeout)
 
 
 def bit(n,d):
@@ -413,15 +419,15 @@ class waveform:
         try:
             npos=data.index("#")
         except ValueError:
-            raise ValueError,"invalid format"            
+            raise ValueError("invalid format")
         header=data[:npos]
         data=data[npos:]
         self.dsize=int(data[2:][:int(data[1])])
         wfdata=data[2:][int(data[1]):]
         if (len(wfdata) < self.dsize):
-            print "Not Enough Data:",
-            print data[:12],self.dsize, len(wfdata)
-            #raise exceptions.ValueError,"Not enough data"
+            sys.stderr.write(b"Not Enough Data:")
+            sys.stderr.write(b"%s %d %d\n"%(data[:12],self.dsize ,len(wfdata)))
+            #raise ValueError("Not enough data")
         instrument_name=array.array('c',wfdata[76:92]).tostring().strip('\x00')
         cord = struct.unpack('b',wfdata[34])[0]
         endian, endiandouble = endian_set(cord, instrument_name)
